@@ -15,10 +15,9 @@
 // specific language governing permissions and limitations
 // under the License.
 
+use anyhow::anyhow;
 use anyhow::Result;
 use clap::Parser;
-use fuse3::path::Session;
-use fuse3::MountOptions;
 use url::Url;
 
 #[derive(Parser, Debug)]
@@ -45,11 +44,11 @@ async fn main() -> Result<()> {
 
 #[cfg(any(target_os = "linux", target_os = "freebsd"))]
 async fn execute(cfg: Config) -> Result<()> {
-    use std::collections::HashMap;
     use std::env;
     use std::str::FromStr;
 
-    use anyhow::anyhow;
+    use fuse3::path::Session;
+    use fuse3::MountOptions;
     use opendal::Operator;
     use opendal::Scheme;
 
@@ -58,11 +57,7 @@ async fn execute(cfg: Config) -> Result<()> {
     }
 
     let scheme_str = cfg.backend.scheme();
-    let op_args = cfg
-        .backend
-        .query_pairs()
-        .into_owned()
-        .collect::<HashMap<String, String>>();
+    let op_args = cfg.backend.query_pairs().into_owned();
 
     let scheme = match Scheme::from_str(scheme_str) {
         Ok(Scheme::Custom(_)) | Err(_) => Err(anyhow!("invalid scheme: {}", scheme_str)),
@@ -119,7 +114,84 @@ async fn execute(cfg: Config) -> Result<()> {
     Ok(())
 }
 
-#[cfg(not(any(target_os = "linux", target_os = "freebsd")))]
+#[cfg(target_os = "windows")]
+async fn execute(cfg: Config) -> Result<()> {
+    use std::path::PathBuf;
+    use std::str::FromStr;
+
+    use anyhow::Context;
+    use cloud_filter::root::HydrationType;
+    use cloud_filter::root::PopulationType;
+    use cloud_filter::root::SecurityId;
+    use cloud_filter::root::Session;
+    use cloud_filter::root::SyncRootIdBuilder;
+    use cloud_filter::root::SyncRootInfo;
+    use opendal::Operator;
+    use opendal::Scheme;
+    use tokio::runtime::Handle;
+    use tokio::signal;
+
+    const PROVIDER_NAME: &str = "ofs";
+
+    if cfg.backend.has_host() {
+        log::warn!("backend host will be ignored");
+    }
+
+    let scheme_str = cfg.backend.scheme();
+    let op_args = cfg.backend.query_pairs().into_owned();
+
+    let scheme = match Scheme::from_str(scheme_str) {
+        Ok(Scheme::Custom(_)) | Err(_) => Err(anyhow!("invalid scheme: {}", scheme_str)),
+        Ok(s) => Ok(s),
+    }?;
+    let backend = Operator::via_iter(scheme, op_args).context("invalid arguments")?;
+
+    let sync_root_id = SyncRootIdBuilder::new(PROVIDER_NAME)
+        .user_security_id(
+            SecurityId::current_user().expect("get current user security id, it might be a bug"),
+        )
+        .build();
+
+    if !sync_root_id
+        .is_registered()
+        .expect("check if sync root is registered, it might be a bug")
+    {
+        sync_root_id
+            .register(
+                SyncRootInfo::default()
+                    .with_display_name(format!("ofs ({scheme_str})"))
+                    .with_hydration_type(HydrationType::Full)
+                    .with_population_type(PopulationType::Full)
+                    .with_icon("%SystemRoot%\\system32\\charmap.exe,0")
+                    .with_version(env!("CARGO_PKG_VERSION"))
+                    .with_recycle_bin_uri("http://cloudmirror.example.com/recyclebin") // FIXME
+                    .unwrap()
+                    .with_path(&cfg.mount_path)
+                    .context("mount_path is not a folder")?,
+            )
+            .context("failed to register sync root")?;
+    }
+
+    let handle = Handle::current();
+    let connection = Session::new()
+        .connect_async(
+            &cfg.mount_path,
+            cloudfilter_opendal::CloudFilter::new(backend, PathBuf::from(&cfg.mount_path)),
+            move |f| handle.clone().block_on(f),
+        )
+        .context("failed to connect to sync root")?;
+
+    signal::ctrl_c().await.unwrap();
+
+    drop(connection);
+    sync_root_id
+        .unregister()
+        .context("failed to unregister sync root")?;
+
+    Ok(())
+}
+
+#[cfg(not(any(target_os = "linux", target_os = "freebsd", target_os = "windows")))]
 async fn execute(_cfg: Config) -> Result<()> {
-    Err(anyhow::anyhow!("platform not supported"))
+    Err(anyhow!("platform not supported"))
 }
